/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Mon Dec 16 11:42:34 PST 2002
    copyright            : (C) 2002 by Oleksiy Kohany
    email                : kohany@girinst.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
#include <string.h>
#include "functions.h"
#include "BioSequence.h"
#include "constants.h"
using namespace std;
class rc;

unsigned int siPos;
/**
======================================================
 Class rc - one record into conversion map.
======================================================
*/
class rc {
public:
	rc () {
		SRC = DST = "";
		iSF = iST = iDF = iDT = 0;
	}
	rc(const string src, unsigned long isf, unsigned long ist, const string dst, unsigned long idf, unsigned long idt) {
		assign(src, isf, ist, dst, idf, idt);
	}
	void assign(const string src, unsigned long isf, unsigned long ist,
		const string dst, unsigned long idf, unsigned long idt) {
		SRC = src;
		DST = dst;
		iSF = isf;
		iST = ist;
		iDF = idf;
		iDT = idt;
	};
  /** No descriptions */
  void print(ostream* os) const {
    if(iSF + iST + iDF + iDT > 0) {
      (*os) << DST << "\t" << iDF << "\t" << iDT << "\t" << SRC << "\t" << iSF << "\t" << iST << "\n";
    }
  }
  /** No descriptions */
  void print(FILE* os) const {
    if(iSF + iST + iDF + iDT > 0) {
      fprintf(os, "%s\t%ld\t%ld\t%s\t%ld\t%ld\n", DST.c_str(), iDF, iDT, SRC.c_str(), iSF, iST);
    }
  }

	string SRC;
	string DST;
	unsigned long iSF;
	unsigned long iST;
	unsigned long iDF;
	unsigned long iDT;
};
//=============================================================================
#ifdef IOSTREAM
void formatedPrint(ostream* os, string& s, unsigned int iLineLen = LINELENGTH) {
	unsigned long iPos;
	*os << s.substr(0, iLineLen - siPos) << "\n";
	for(iPos = iLineLen - siPos; iPos < s.size(); iPos += iLineLen) {
		*os << s.substr(iPos, iLineLen);
		siPos = s.size() - iPos;
		if(siPos > iLineLen) {
			*os << "\n";
		}
	}
}
#else
void formatedPrint(FILE* os, string& s, unsigned int iLineLen = LINELENGTH) {
	unsigned long iPos;
	fprintf(os, "%s\n", s.substr(0, iLineLen - siPos).c_str());
	for(iPos = iLineLen - siPos; iPos < s.size(); iPos += iLineLen) {
		fprintf(os, "%s", s.substr(iPos, iLineLen).c_str());
		siPos = s.size() - iPos;
		if(siPos > iLineLen) {
			fprintf(os, "\n");
		}
	}
}
#endif
//=============================================================================
#ifdef IOSTREAM
void sjoin(istream* is, ostream* os, ostream* smap, unsigned long iMNSZ, unsigned long iNSZ) {
#else
void sjoin(FILE* is, FILE* os, FILE* smap, unsigned long iMNSZ, unsigned long iNSZ) {
#endif
  BioSequence seq1;
  BioSequence seq2;
  char sLine[MAXLINELENGTH];
  char sName[MAXNAMESIZE];
  char sNewName[MAXNAMESIZE];
  rc row;
  unsigned int fcnt = 0;
#ifdef IOSTREAM
  while(!is->getline(sLine, MAXLINELENGTH).fail()) {
#else
    while(fgets(sLine, MAXLINELENGTH, is) != 0) {
#endif
      if(sLine[0] == '>' || sLine[0] == ';') {
        if(seq2.size() > 0) {
          if(fcnt > 0) {
            formatedPrint(os, seq2);
            fcnt++;
            seq2.clear();
          }
          else {
            if(seq1.size() == 0) {
              sprintf(sNewName, "%s_%p", seq2.getName().c_str(), sNewName);
              seq1.setName(sNewName);
            }
            else {
              seq1.append("-");
            }
            row.iDF = seq1.size() + 1;
            seq1.append(seq2);
            row.DST = seq1.getName();
            row.iST = seq2.size();
            row.iDT = seq1.size();
            row.print(smap);
            seq2.clear();
            if(seq1.size() >= iNSZ) {
              seq1.print(os);
              seq1.clear();
            }
          }
        }
        if(sLine[0] == '>') {
          sscanf(sLine + 1, "%s", sName);
        }
        if(sLine[0] == ';') {
#ifdef IOSTREAM
          while(!is->getline(sLine, MAXLINELENGTH).eof() && sLine[0] == ';');
#else
          while(fgets(sLine, MAXLINELENGTH, is) != 0 && sLine[0] == ';');
#endif
          sscanf(sLine, "%s", sName);
        }
        if(fcnt > 0) {
          fcnt = 0;
          siPos = 0;
#ifdef IOSTREAM
          *os << "\n";
#else
          fprintf(os, "\n");
#endif
        }
        row.assign(sName, 1, 0, sName, 1, 0);
        seq2.setName(sName);
        continue;
      }
      char *pcPtr = sLine + strlen(sLine);
      if(*(--pcPtr) == '\n') { *(pcPtr--) = '\0'; }
      if(*(pcPtr) == '1' ) { *pcPtr = '\0'; }
      seq2.append(sLine);
      if(seq2.size() >= iMNSZ) {
        if(fcnt == 0) {
          seq2.print(os);
        }
        else {
          formatedPrint(os, seq2);
        }
        fcnt++;
        seq2.clear();
      }
    }
    if(seq2.size() > 0) {
      if(fcnt > 0) {
        formatedPrint(os, seq2);
#ifdef IOSTREAM
        *os << "\n";
#else
        fprintf(os, "\n");
#endif
        seq2.clear();
      }
    }
    if(seq1.size() > 0) {
      if(seq2.size() > 0) {
        seq1.append("-");
        row.iDF = seq1.size() + 1;
        seq1.append(seq2);
        row.DST = seq1.getName();
        row.iST = seq2.size();
        row.iDT = seq1.size();
        row.print(smap);
        seq2.clear();
      }
      seq1.print(os);
    }
    else {
      if(seq2.size() > 0) {
        seq2.print(os);
      }
    }
  }
//=============================================================================
void usage(char* exec) {
   if(!exec) {
      return;
   }
   if(strrchr(exec, '/')) {
      exec = strrchr(exec, '/') + 1;
   }
   cerr << "NAME\n\t" << exec;
#ifdef VERSION
   cerr << " V" << VERSION;
#endif
   cerr << " - joins sequences\n";
   cerr << "\nSYNOPSIS\n\t" << exec << " [infile] mmap normsize minsize [outfile]\n\n";
   cerr << "DESCRIPTION\n\tJoins database sequences shorter than minsize into sequences\n";
   cerr << "\tof size longer or equal to normsize.\n\n";
   cerr << "\tmmap - is a backward conversion map.\n";
#ifdef COPYRIGHT
   cerr << "\nCOPYRIGHT\n\t" << COPYRIGHT << "\n";
#endif
}
//=============================================================================
int main(int argc, char *argv[])
{
#ifdef IOSTREAM
	istream *is;
	ostream *os;
	ofstream *ofs;
#else
  FILE* is;
  FILE* os;
  FILE* ofs;
#endif
	char** args = argv;
	unsigned long iMNSZ, iNSZ;
	if((args = assigninput(is, args)) == 0) {
		usage(argv[0]);
		exit(10);
	}
#ifdef IOSTREAM
	if((ofs = openofs(args[1])) == 0) {
#else
	if((ofs = openofilename(args[1])) == 0) {
#endif
		usage(argv[0]);
		exit(20);
	}
	args++;
	if(args[1] == 0 || sscanf(args[1], "%ld", &iNSZ) < 1) {
	usage(argv[0]);
		exit(30);
	}
	args++;
	if(args[1] == 0 || sscanf(args[1], "%ld", &iMNSZ) < 1) {
		usage(argv[0]);
		exit(40);
	}
	args++;
	if((args = assignoutput(os, args)) == 0) {
		usage(argv[0]);
		exit(50);
	}
	if(iMNSZ > iNSZ) {
		cerr << "'minsize' has to be less than 'normsize'.\n";
		exit(60);
	}
//========================================================================
	sjoin(is, os, ofs, iMNSZ, iNSZ);
//========================================================================
#ifdef IOSTREAM
	os->flush();
  ofs->close();
#else
  fflush(os);
  fclose(ofs);
#endif
	return 0;
}
